package net.nosial.socialclient;

import net.nosial.socialclient.abstracts.RpcResult;
import net.nosial.socialclient.classes.Cryptography;
import net.nosial.socialclient.classes.RpcClient;
import net.nosial.socialclient.classes.Utilities;
import net.nosial.socialclient.exceptions.CryptographyException;
import net.nosial.socialclient.exceptions.ResolutionException;
import net.nosial.socialclient.exceptions.RpcException;
import net.nosial.socialclient.objects.RpcRequest;

import java.security.KeyPair;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.HashMap;
import java.util.Map;

public class Client extends RpcClient
{
    protected Client(String domain) throws ResolutionException, CryptographyException
    {
        super(domain);
    }

    public Client(String endpoint, PublicKey serverPublicKey)
    {
        super(endpoint, serverPublicKey);
    }

    public String createSession(KeyPair keyPair) throws RpcException
    {
        RpcResult response = this.sendRequest(new RpcRequest("create_session", Utilities.randomCrc32(), new HashMap<>(){{
            put("public_key", Cryptography.exportPublicKey(keyPair));
        }}));

        if(response == null)
        {
            throw new RpcException("Response is null");
        }

        if(!response.isSuccess())
        {
            throw new RpcException(response);
        }

        return (String) response.getResponse().getResult();
    }

    public void setSession(String sessionUuid, PrivateKey privateKey)
    {
        this.sessionUuid = sessionUuid;
        this.privateKey = privateKey;
    }

    public String createAndSetSession(KeyPair keyPair) throws RpcException
    {
        String sessionUuid = this.createSession(keyPair);
        this.setSession(sessionUuid, keyPair.getPrivate());

        return sessionUuid;
    }

    public boolean ping() throws RpcException
    {
        RpcResult response = this.sendRequest(new RpcRequest("ping", Utilities.randomCrc32()));

        if(response == null)
        {
            throw new RpcException("Response is null");
        }

        if(!response.isSuccess())
        {
            throw new RpcException(response);
        }

        return true;
    }
}
